local CS = LibStub("AceAddon-3.0"):GetAddon("Conspicuous Spirits", true)
if not CS then return end

local EF = CS:GetModule("EncounterFixes")


---------------------
-- Encounter fixes --
---------------------

-- Flamebender
EF:RegisterEncounter(1689, function()
	EF:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", function(_, timeStamp, event, _, sourceGUID, _, _, _, destGUID, destName, _, _, spellID)
		if spellID == 181089 then  -- "Encounter Event" (when wolves vanish)
			CS:RemoveGUID(sourceGUID)
		end
	end)
end)

-- Blast Furnace
EF:RegisterEncounter(1690, function()
	EF:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", function(_, timeStamp, event, _, sourceGUID, _, _, _, destGUID, destName, _, _, spellID)
		if spellID == 605 and event == "SPELL_CAST_SUCCESS" then  -- Dominate Mind
			CS:RemoveGUID(destGUID)  -- possibly replace with HideGUID
		end
	end)
end)

-- Hans'gar and Franzok
EF:RegisterEncounter(1693, function()
	EF:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT", function()
		local boss1GUID = UnitGUID("boss1")
		local boss2GUID = UnitGUID("boss2")
		if boss2GUID and boss1GUID then
			EF:UnregisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")
			EF:RegisterEvent("UNIT_TARGETABLE_CHANGED", function()
				if not UnitExists("boss1") then
					CS:RemoveGUID(boss1GUID)  -- possibly replace with HideGUID
				
				elseif not UnitExists("boss2") then
					CS:RemoveGUID(boss2GUID)  -- possibly replace with HideGUID
				
				end
			end)
		end
	end)
end)

-- Beastlord Darmac
EF:RegisterEncounter(1694, function()
	EF:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT", function()
		-- check when Beastlord becomes untargetable due to mounting up
		local darmacGUID = UnitGUID("boss1")
		if darmacGUID then
			EF:UnregisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")
			EF:RegisterEvent("UNIT_TARGETABLE_CHANGED", function()
				if not UnitExists("boss1") then
					CS:RemoveGUID(darmacGUID)  -- possibly replace with HideGUID
				end
			end)
		end
	end)
	-- search for overkill since there isn't always a death event for the spears
	EF:CheckForOverkill()
end)

-- Oregorger
EF:RegisterEncounter(1696, function()
	-- missing death events on Ore Crates
	EF:CheckForOverkill()
end)

-- Blackhand
EF:RegisterEncounter(1704, function()
	-- fix for disappearing Siegemakers in P2 -> P3 transition
	local bossGUID = {}
	EF:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT", function()
		for i = 1, 5 do
			local unitID = "boss"..tostring(i)
			local GUID = UnitGUID(unitID)
			-- don't overwrite with nil because INSTANCE_ENCOUNTER_ENGAGE_UNIT fires before COMBAT_LOG_EVENT_UNFILTERED
			bossGUID[i] = GUID or bossGUID[i]
		end
	end)
	EF:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", function(_, timeStamp, event, _, sourceGUID, _, _, _, destGUID, destName, _, _, spellID)
		if spellID == 177487 then  -- Shattered Floor
			-- also doable with 161348 (Jump To Third Floor) UNIT_SPELLCAST_SUCCEEDED
			
			-- debug
			CS:Debug("Shattered Floor")
			
			for i = 1, 5 do
				local GUID = bossGUID[i]
				if GUID and (EF:GetNPCID(GUID) ~= 77325) then  -- Blackhand
					
					-- debug
					CS:Debug("Removing "..tostring(GUID))
					
					CS:RemoveGUID(GUID)
				end			
			end
			EF:UnregisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")
			-- no point in keeping event registered in case player is still on balcony in transition since Falling Damage won't trigger then
			-- also Shattered Floor events get spammed
			EF:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			
		elseif spellID == "Falling" and destGUID == UnitGUID("player") then
			-- aimed at taking fall damage after jumping from balcony
			-- for this to work you need to be bad and not use Levitate when jumping from the balcony
			CS:RemoveAllGUIDsExcept(bossGUID[1], bossGUID[2], bossGUID[3], bossGUID[4], bossGUID[5])
			
		end
	end)
end)